<?php

/**
 * @file
 * Link apache solr search functionalities to services module.
 * See services module documentatioN: https://www.drupal.org/project/services
 */

/**
 * Callback for solr service.
 * See solr_service_services_resources().
 */
function _solr_service_retrieve($fn = 'retrieve', $search_page_id = 'core_search', $key = NULL, $filters = NULL, $fields = NULL, $facets = FALSE, $page = 0) {
  $results = array();

  // @TODO: admin setting to control logging of debug messages.
  // watchdog('solr_service', 'search.solr invoked for !keys', array('!keys' => $key), WATCHDOG_DEBUG);

  try {
    // Load search page conditions.
    $search_page = apachesolr_search_page_load($search_page_id);
    $conditions = apachesolr_search_conditions_default($search_page);

    // Apply filters (if set).
    // Should be a comma-separated list of Lucene filter queries to apply by default. E.g. "bundle:blog,is_uid:(1 OR 2)".
    if (!empty($filters)) {
      $filters = explode(',', $filters);
      foreach ($filters as $filter) {
        $conditions['fq'][]  = $filter;
      }
    }

    // Execute the search.
    $results = apachesolr_search_search_execute($key, $conditions);
  }
  catch (Exception $e) {
    watchdog('solr_service', $e->getMessage(), NULL, WATCHDOG_ERROR);
    apachesolr_failure(t('Solr search'), $key);
  }

  // @TODO: admin setting to control logging of debug messages.
  // watchdog('solr_service', 'search.solr returned !count results for !keys', array('!count' => count($results), '!keys' => $key), WATCHDOG_DEBUG);

  if ($results and is_array($results) and count($results)) {
    // If specific fields are requested, remove extra data.
    // Note that doing this takes a few fractions of a millisecond in PHP, but probably
    // worth it to speed up network time, and json results processing.
    // @TODO: would maybe be preferable to alter the solr query directly, so that
    // only desired fields are returned in the first place.
    if ($fields) {
      $stdkeys = explode(',', str_replace(' ', '', $fields));
      $results_fields = array();
      foreach ($results as $rk => $r) {
        // Loop through top-level fields.
        foreach ($r as $k => $v) {
          if (in_array($k, $stdkeys)) {
            $results_fields[$rk][$k] = $v;
          }
        }
        // Loop through 'fields' array.
        foreach ($r['fields'] as $k => $v) {
          if (in_array($k, $stdkeys)) {
            $results_fields[$rk][$k] = $v;
          }
        }
      }
      $results = $results_fields;
      unset($results_fields);
    }

    // Return total result count.
    // @TODO: is this the most efficient way to get the count?
    $env_id = apachesolr_default_environment();
    $query = apachesolr_current_query($env_id);
    if (is_object($query)) {
      $response = apachesolr_static_response_cache($query->getSearcher());
      $results['total'] = $response->response->numFound;
    }

    // Return facets, if desired.
    if ($facets) {
      $searcher = $query->getSearcher();
      $results['facets'] = facetapi_build_realm($searcher, 'block');
    }
  }
  else {
    // No results. Return empty array.
    $results = array();

    // Can alternatively return a services error using the code below.
    // module_load_include('inc', 'services', 'includes/services.runtime');
    // return services_error(t('Search returned no results.'));
  }

  return $results;
}